#include "runtime.h"
#include "config.h"
#include "aio.h"
#include "file_tab.h"
#include "buffer_manager.h"
#include "uidtab.h"

extern vy_cvar rt_Renderer;
extern vy_cvar rt_Fullscreen;
extern vy_cvar rt_WindowWidth;
extern vy_cvar rt_WindowHeight;
extern vy_cvar rt_BufferManagerMemory;

void __RegisterRuntimeCVars(void) {
    vyRegisterCVAR(&rt_Renderer);
    vyRegisterCVAR(&rt_Fullscreen);
    vyRegisterCVAR(&rt_WindowWidth);
    vyRegisterCVAR(&rt_WindowHeight);
    vyRegisterCVAR(&rt_BufferManagerMemory);
}

extern void __RTSetMainThreadId(void);

VY_DLLEXPORT vy_result vyInitRuntime(void) {
    __RTSetMainThreadId();
    __RegisterRuntimeCVars();

    vy_result res;
    if ((res = vyInitBufferManager()) != VY_SUCCESS) {
        vyReportError("BUFFERMGR", "Init failed.");
        return res;
    }

    if ((res = vyInitFileTab(1024)) != VY_SUCCESS) {
        vyReportError("FTAB", "Init failed.");
        return res;
    }

    if ((res = vyInitAIO(0)) != VY_SUCCESS) {
        vyReportError("AIO", "Init failed.");
        return res;
    }

    if ((res = vyLoadUIDTable()) != VY_SUCCESS) {
        vyLog("CORE", "LoadUIDTable returned result: %u", res);
    }

    return VY_SUCCESS;
}

VY_DLLEXPORT void vyShutdownRuntime(void) {
    vyReleaseUIDTable();
    vyShutdownAIO();
    vyShutdownFileTab();
    vyShutdownBufferManager();
}